//--------------------------------------
// VST Plug-Ins SDK 
// BALANCE
//
// Francisco Javier Ripoll Esteve
// ripollete@hotmail.com
// 13/05/2008
//--------------------------------------

#include "balance.h"


AudioEffect* createEffectInstance (audioMasterCallback audioMaster)
{
	return new ABalance (audioMaster);
}


ABalance::ABalance (audioMasterCallback audioMaster) : AudioEffectX (audioMaster, 1, 1)	{
	setNumInputs (2);		// stereo in
	setNumOutputs (2);		// stereo out
	setUniqueID ('Bal');	// Identificador
	canProcessReplacing ();	
	canDoubleReplacing ();	

	fBalance = 0;			
}


ABalance::~ABalance () {

}


void ABalance::setParameter (VstInt32 index, float value) {
	fBalance = (value*200) -100;

}


float ABalance::getParameter (VstInt32 index) {
	float v;
	v=(fBalance+100.f)/200.f;
	return v;
}


void ABalance::getParameterName (VstInt32 index, char* label) {
	switch (index) {
		case NumeroProgramBalance : vst_strncpy (label, "Balance", kVstMaxParamStrLen);
			break;
	}
}


void ABalance::getParameterDisplay (VstInt32 index, char* text) {
	float2string (fBalance, text, kVstMaxParamStrLen);    
}


void ABalance::getParameterLabel (VstInt32 index, char* label) {
	vst_strncpy (label, "%", kVstMaxParamStrLen);  
}


bool ABalance::getEffectName (char* name) {
	vst_strncpy (name, "Balance", kVstMaxEffectNameLen);
	return true;
}


bool ABalance::getProductString (char* text) {
	vst_strncpy (text, "Balance", kVstMaxProductStrLen);
	return true;
}


bool ABalance::getVendorString (char* text) {
	vst_strncpy (text, "Francisco Javier Ripoll", kVstMaxVendorStrLen);
	return true;
}


VstInt32 ABalance::getVendorVersion () { 
	return 1000; 
}


void ABalance::processReplacing (float** inputs, float** outputs, VstInt32 sampleFrames) {
    float* in1  =  inputs[0];
    float* in2  =  inputs[1];
    float* out1 = outputs[0];
    float* out2 = outputs[1];

    while (--sampleFrames >= 0)
    {
		if(fBalance==0.0) {
			(*out1++) = (*in1++);
			(*out2++) = (*in2++);
		}
		
		if(fBalance>0.0) {  //Dreta
			float aux=fBalance/100;
			(*out1++) = (*in1++)*(1-aux);
			(*out2++) = (*in2++);
		}

		if(fBalance<0.0) {  //Esquerra
			float aux=fBalance/100;
			aux= -aux;
			(*out1++) = (*in1++);
			(*out2++) = (*in2++)*(1-aux);
		}


    }
}


void ABalance::processDoubleReplacing (double** inputs, double** outputs, VstInt32 sampleFrames)
{
    double* in1  =  inputs[0];
    double* in2  =  inputs[1];
    double* out1 = outputs[0];
    double* out2 = outputs[1];

    while (--sampleFrames >= 0)
    {
		if(fBalance==0.0) {
			(*out1++) = (*in1++);
			(*out2++) = (*in2++);
		}
		
		if(fBalance>0.0) {  //Dreta
			double aux= fBalance/100.0;
			(*out1++) = (*in1++)*(1-aux);
			(*out2++) = (*in2++);
		}

		if(fBalance>0.0) {  //Esquerra
			double aux=fBalance/100.0;
			aux= -aux;
			(*out1++) = (*in1++);
			(*out2++) = (*in2++)*(1-aux);
		}


    }
}