//--------------------------------------
// VST Plug-Ins SDK 
// CHORUS
//
// Francisco Javier Ripoll Esteve
// ripollete@hotmail.com
// 13/05/2008
//--------------------------------------

#include "chorus.h"
#include "math.h"


AudioEffect* createEffectInstance (audioMasterCallback audioMaster) {
	return new AChorus (audioMaster);
}


AChorus::AChorus (audioMasterCallback audioMaster) : AudioEffectX (audioMaster, 1, 5)	{
	L = 44100;
	sampleCount = 0;
	buffer1 = new float[L];
	buffer2 = new float[L];
	buffer1d = new double[L];
	buffer2d = new double[L];

	//inicialitzem els buffers
	for (int i=0;i<L;i++) {
		buffer1[i] = 0.f;
		buffer2[i] = 0.f;
		buffer1d[i] = 0;
		buffer2d[i] = 0;
	}

	fGain=  1.f;
	fFreq = 2.5f;
	fDelayMin = 0.2f;
	fDelayMax = 0.3f;
	fDryWet = 0.5;
	n=0;

	posicion = 0;

	delayMin = int (L*fDelayMin);
	delayMax = int (L*fDelayMax);


	setNumInputs (2);	// stereo input
	setNumOutputs (2);	// stereo output

	setUniqueID ('Acho');

	resume ();
}


AChorus::~AChorus () {
	if (buffer1)
		delete[] buffer1;

	if (buffer2)
		delete[] buffer2;
	
	if (buffer1d)
		delete[] buffer1d;

	if (buffer2d)
		delete[] buffer2d;
}

void AChorus::setDelayMin (float value)  {
	 fDelayMin = value;
	 delayMin= int(value*L);
}

void AChorus::setDelayMax (float value) {
	 fDelayMax = value;
	 delayMax= int(value*L);
}

void AChorus::resume () {
	memset (buffer1, 0, L * sizeof (float));
	memset (buffer2, 0, L * sizeof (float));
	AudioEffectX::resume ();
}

void AChorus::setParameter (VstInt32 index, float value) {
	
	switch (index) 	{				
		case kDryWet   : fDryWet = value;		break;
		case kGain     : fGain = value*2;	    break;
		case kFreq     : fFreq = value * 5.f;   break;
		case kDelayMin : setDelayMin(value);    break;
		case kDelayMax : setDelayMax(value);    break;
	}
}


float AChorus::getParameter (VstInt32 index) {
	float v = 0;

	switch (index) 	{
		case kDryWet   : v = fDryWet;										break;
		case kGain     : v = fGain/2;										break;
		case kFreq     : v = fFreq / 5.f;									break;
		case kDelayMin : v = fDelayMin;										break;
		case kDelayMax : v = fDelayMax;										break;
	}
	return v;
}

void AChorus::getParameterName (VstInt32 index, char* label) {
	switch (index) 	{
		case kDryWet   : strcpy (label, "Dry -- Wet");	break;
		case kGain     : strcpy(label, "Ganancia");		break;
		case kFreq     : strcpy(label, "Dealy Freq");	break;
		case kDelayMin : strcpy (label, "DelayMin");	break;
		case kDelayMax : strcpy (label, "DelayMax");	break;
	}
}

void AChorus::getParameterDisplay (VstInt32 index, char* text) {
		switch (index) 	{
		case kDryWet   : float2string (fDryWet*100, text, kVstMaxParamStrLen);		break;
		case kGain     : dB2string(fGain, text, kVstMaxProgNameLen);				break;
		case kFreq     : float2string(fFreq, text, kVstMaxProgNameLen);				break;
		case kDelayMin : float2string (fDelayMin*1000, text, kVstMaxParamStrLen);	break;
		case kDelayMax : float2string (fDelayMax*1000, text, kVstMaxParamStrLen);	break;
	}
}

void AChorus::getParameterLabel (VstInt32 index, char* label) {
	switch (index) 	{
		case kDryWet   : strcpy (label, "%");		break;
		case kGain     : strcpy(label, "dB");		break;
		case kFreq     : strcpy(label, "Hz");		break;
		case kDelayMin : strcpy (label, "ms");		break;
		case kDelayMax : strcpy (label, "ms");		break;
	}
}

bool AChorus::getEffectName (char* name) {
	strcpy (name, "Delay");
	return true;
}

bool AChorus::getProductString (char* text) {
	strcpy (text, "Delay");
	return true;
}


bool AChorus::getVendorString (char* text) {
	strcpy (text, "Francisco Javier Ripoll Esteve");
	return true;
}

void AChorus::processReplacing (float** inputs, float** outputs, VstInt32 sampleFrames) {

	
	float *in1  =  inputs[0];
    float *in2  =  inputs[1];
    float *out1 =  outputs[0];
    float *out2 =  outputs[1];
	float T = 1/44100;
	int di=0;
	double pi = 3.1415926;
	double nT=0, deltai=0, t0=0, t1=0, t2=0, t3=0, b0=0, b1=0, b2=0, b3=0;

    while (--sampleFrames >= 0) {
		nT = 1/44100 * (n++);
		deltai = (delayMax-delayMin)/2 + delayMin + (delayMax-delayMin)/2 * sin(2 * pi * fFreq * nT);
		// Delay line emprant Lagrange interpolator
		di = floorf(deltai) - 1;
		t0 = deltai - di;
		t1 = t0 - 1;
		t2 = t0 - 2;
		t3 = t0 - 3;
		b0 = -t1*t2*t3/6;
		b1 =  t0*t2*t3/2;
		b2 = -t0*t1*t3/2;
		b3 =  t0*t1*t2/6;

		
		if (posicion >= L)
			posicion = 0;

		//Esquerra
		buffer1[posicion] = fGain*(*in1);
		for (int i=0;i<4;i++) {
			outPoint[i] = (posicion - 1 - di + i);
			if (outPoint[i] < 0)
				outPoint[i] += L;
			else if (outPoint[i] >= L)
				outPoint[i] -= L;
			listate[i] = buffer1[outPoint[i]];
		}

		feedbackL = (b0*listate[0] + b1*listate[1] + b2*listate[2] + b3*listate[3]);

		
		//Dreta
		buffer2[posicion] = fGain*(*in2);
		for (int j=0;j<4;j++) {
			outPoint[j] = (posicion - 1 - di + j);
			if (outPoint[j] < 0)
				outPoint[j] += L;
			if (outPoint[j] >= L)
				outPoint[j] -= L;
			listate[j] = buffer2[outPoint[j]];
		}
		
		feedbackR = (b0*listate[0] + b1*listate[1] + b2*listate[2] + b3*listate[3]);

		// ixida
		(*out1++) = (*in1++)*fDryWet + (1-fDryWet) * feedbackL;
		(*out2++) = (*in2++)*fDryWet + (1-fDryWet) * feedbackR;

		posicion++;

   }
}

void AChorus::processDoubleReplacing (double** inputs, double** outputs, VstInt32 sampleFrames) {
	
	double *in1  =  inputs[0];
    double *in2  =  inputs[1];
    double *out1 =  outputs[0];
    double *out2 =  outputs[1];
	double T = 1/44100;
	int di=0;
	double pi = 3.1415926;
	double nT=0, deltai=0, t0=0, t1=0, t2=0, t3=0, b0=0, b1=0, b2=0, b3=0;

    while (--sampleFrames >= 0) {
		nT = 1/44100 * (n++);
		deltai = (delayMax-delayMin)/2 + delayMin + (delayMax-delayMin)/2 * sin(2 * pi * fFreq * nT);
		// Delay line emprant Lagrange interpolator
		di = floorf(deltai) - 1;
		t0 = deltai - di;
		t1 = t0 - 1;
		t2 = t0 - 2;
		t3 = t0 - 3;
		b0 = -t1*t2*t3/6;
		b1 =  t0*t2*t3/2;
		b2 = -t0*t1*t3/2;
		b3 =  t0*t1*t2/6;

		
		if (posicion >= L)
			posicion = 0;

		//Esquerra
		buffer1[posicion] = fGain*(*in1);
		for (int i=0;i<4;i++) {
			outPoint[i] = (posicion - 1 - di + i);
			if (outPoint[i] < 0)
				outPoint[i] += L;
			else if (outPoint[i] >= L)
				outPoint[i] -= L;
			listate[i] = buffer1[outPoint[i]];
		}

		feedbackL = (b0*listate[0] + b1*listate[1] + b2*listate[2] + b3*listate[3]);

		
		//Dreta
		buffer2[posicion] = fGain*(*in2);
		for (int j=0;j<4;j++) {
			outPoint[j] = (posicion - 1 - di + j);
			if (outPoint[j] < 0)
				outPoint[j] += L;
			if (outPoint[j] >= L)
				outPoint[j] -= L;
			listate[j] = buffer2[outPoint[j]];
		}
		
		feedbackR = (b0*listate[0] + b1*listate[1] + b2*listate[2] + b3*listate[3]);

		// ixida
		(*out1++) = (*in1++)*fDryWet + (1-fDryWet) * feedbackL;
		(*out2++) = (*in2++)*fDryWet + (1-fDryWet) * feedbackR;

		posicion++;

   }
}
