//--------------------------------------
// VST Plug-Ins SDK 
// DELAY
//
// Francisco Javier Ripoll Esteve
// ripollete@hotmail.com
// 13/05/2008
//--------------------------------------

#include "delay.h"
#include "math.h"


AudioEffect* createEffectInstance (audioMasterCallback audioMaster) {
	return new ADelay (audioMaster);
}


ADelay::ADelay (audioMasterCallback audioMaster) : AudioEffectX (audioMaster, 1, 2) {
	L = 8820;     //Un segon 44100, 200 millisegons 8820  
	buffer1 = new float[L];
	buffer2 = new float[L];
	buffer1d = new double[L];
	buffer2d = new double[L];
		
	fDelay = 0.1f;
	fDryWet = 0;

	posicion = 0;

	delay = int(L*fDelay);


	setNumInputs (2);	// stereo input
	setNumOutputs (2);	// stereo output

	setUniqueID ('ADly');

	resume ();		// flush buffer
}


ADelay::~ADelay () {
	if (buffer1)
		delete[] buffer1;

	if (buffer2)
		delete[] buffer2;
}



void ADelay::setDelay (float value)  {
	 fDelay = float(0.001*exp(value*log(0.200/0.001)));
	 delay= int(fDelay*L);
}


//void ADelay::setProgramName (char* name) {
	//vst_strncpy (programName, name, kVstMaxProgNameLen);
//}


void ADelay::resume () {
	memset (buffer1, 0, L * sizeof (float));
	memset (buffer2, 0, L * sizeof (float));
	AudioEffectX::resume ();
}


void ADelay::setParameter (VstInt32 index, float value) {	
	switch (index) 	{
		case kDelay  : setDelay(value);  break;				
		case kDryWet : fDryWet = (value*100); break;
	}
}


float ADelay::getParameter (VstInt32 index) {
	float v = 0;
	switch (index) 	{
		case kDelay  : v = float(log(fDelay/0.001)/(log(0.200/0.001)));	break;
		case kDryWet : v = fDryWet/100; break;
	}
	return v;
}


void ADelay::getParameterName (VstInt32 index, char* label)
{
	switch (index) {
		case kDelay  : strcpy (label, "Delay");		break;
		case kDryWet : strcpy (label, "Dry -- Wet");	break;
	}
}


void ADelay::getParameterDisplay (VstInt32 index, char* text)
{
		switch (index) {
			case kDelay  : float2string (fDelay, text, kVstMaxParamStrLen);		break;
			case kDryWet : float2string (fDryWet, text, kVstMaxParamStrLen);	break;
	}
}


void ADelay::getParameterLabel (VstInt32 index, char* label) {
	switch (index) {
		case kDelay  : strcpy (label, "segons");	break;
		case kDryWet : strcpy (label, "%");			break;
	}
}


bool ADelay::getEffectName (char* name) {
	strcpy (name, "Delay");
	return true;
}


bool ADelay::getProductString (char* text) {
	strcpy (text, "Delay");
	return true;
}


bool ADelay::getVendorString (char* text) {
	strcpy (text, "Francisco Javier Ripoll Esteve");
	return true;
}


void ADelay::processReplacing (float** inputs, float** outputs, VstInt32 sampleFrames) {
	float* in1 = inputs[0];
	float* in2 = inputs[1];
	float* out1 = outputs[0];
	float* out2 = outputs[1];

	while (--sampleFrames >= 0)	{		

		//***** Guardem les entrades en variables*****//
		float inleft = (*in1++);
		float inright = (*in2++);


		//***** Calcul de la posici on es trobar el frame del retard *****//
		int posicionretardo = posicion-delay;

		if (posicionretardo<0) {
			posicionretardo= L+posicionretardo;
		}

		//**** Reproducci amb el in actual i el buffer(posici-retard) ****//
		*out1++ = float(inleft*(1-(fDryWet/100))+buffer1[posicionretardo]*(fDryWet/100.0));
		*out2++=  float(inright*(1-(fDryWet/100))+buffer2[posicionretardo]*(fDryWet/100.0));

	
		//***** Guardem el in actual en el buffer actual *****//
		buffer1[posicion]=inleft;
		buffer2[posicion]=inright;

	
		posicion++;

		if (posicion==L) {
			posicion=0;
		}
	}
}

//-----------------------------------------------------------------------------------------
void ADelay::processDoubleReplacing (double** inputs, double** outputs, VstInt32 sampleFrames)
{
    double* in1  =  inputs[0];
    double* in2  =  inputs[1];
    double* out1 = outputs[0];
    double* out2 = outputs[1];


	while (--sampleFrames >= 0)	{

		

		//***** Guardem les entrades en variables*****//
		double inleft  = (*in1++);
		double inright = (*in2++);


		//***** Calcul de la posici on es trobar el frame del retard *****//
		int posicionretardo = posicion-delay;

		if (posicionretardo<0) {
			posicionretardo= L+posicionretardo;
		}


		//**** Reproducci amb el in actual i el buffer(posici-retard) ****//
		*out1++ = double(inleft*(1-(fDryWet/100))+buffer1d[posicionretardo]*(fDryWet/100.0));
		*out2++=  double(inright*(1-(fDryWet/100))+buffer2d[posicionretardo]*(fDryWet/100.0));

	
		//***** Guardem el in actual en el buffer actual *****//
		buffer1d[posicion]=inleft;
		buffer2d[posicion]=inright;

	
		posicion++;

		if (posicion==L) {
			posicion=0;
		}
	}
}