//--------------------------------------
// VST Plug-Ins SDK 
// DISTORSI
//
// Francisco Javier Ripoll Esteve
// ripollete@hotmail.com
// 13/05/2008
//--------------------------------------

#include "distorsio.h"


AudioEffect* createEffectInstance (audioMasterCallback audioMaster) {
	return new ADistorsio (audioMaster);
}


ADistorsio::ADistorsio (audioMasterCallback audioMaster): AudioEffectX (audioMaster, 1, 2)	{

	setNumInputs (2);		// stereo in
	setNumOutputs (2);		// stereo out
	setUniqueID ('Dist');	// identify
	canProcessReplacing ();	
	canDoubleReplacing ();

	MaxBits=1.f;  //32767
	MinBits=0.0725f;   //2376

	fGain = 1.f;			// default to 0 dB

	fDistorsio=0.f;
	iDistorsio=MaxBits;
	vst_strncpy (programName, "Default", kVstMaxProgNameLen);	// default program name
}


ADistorsio::~ADistorsio () {

}


void ADistorsio::setProgramName (char* name) {
	vst_strncpy (programName, name, kVstMaxProgNameLen);
}

void ADistorsio::getProgramName (char* name) {
	vst_strncpy (name, programName, kVstMaxProgNameLen);
}


void ADistorsio::setParameter (VstInt32 index, float value) {
	switch (index) 	{
		case kDistorsio :   iDistorsio= MaxBits-value*(MaxBits-MinBits);
							fDistorsio=value*100;
			break;				
		case kGain : fGain = value*2; 
			break;
	}
}

float ADistorsio::getParameter (VstInt32 index) {
	float v = 0;

	switch (index) 	{
		case kDistorsio : v = fDistorsio/100;	break;
		case kGain      : v = fGain/2;			break;
	}
	return v;
}

void ADistorsio::getParameterName (VstInt32 index, char* label) {
		switch (index) {
			case kDistorsio : strcpy (label, "Distorsio");	break;
			case kGain      : strcpy (label, "Ganancia");	break;
	}
}

void ADistorsio::getParameterDisplay (VstInt32 index, char* text) {
	switch (index) {
		case kDistorsio : float2string (fDistorsio, text, kVstMaxParamStrLen);	break;
		case kGain      : dB2string (fGain, text, kVstMaxParamStrLen);			break;
	}
}

void ADistorsio::getParameterLabel (VstInt32 index, char* label) {
	switch (index) 	{
		case kDistorsio : strcpy (label, "poca -- molta");	break;
		case kGain      : strcpy (label, "dB");				break;
	}
}

bool ADistorsio::getEffectName (char* name) {
	vst_strncpy (name, "Distorsio", kVstMaxEffectNameLen);
	return true;
}

bool ADistorsio::getProductString (char* text) {
	vst_strncpy (text, "Distorsio", kVstMaxProductStrLen);
	return true;
}

bool ADistorsio::getVendorString (char* text) {
	vst_strncpy (text, "Francisco Javier Ripoll Esteve", kVstMaxVendorStrLen);
	return true;
}

VstInt32 ADistorsio::getVendorVersion () { 
	return 1000; 
}

void ADistorsio::processReplacing (float** inputs, float** outputs, VstInt32 sampleFrames) {
    float* in1  =  inputs[0];
    float* in2  =  inputs[1];
    float* out1 = outputs[0];
    float* out2 = outputs[1];
	float inleft;
	float inright;
	float distorsio;

    while (--sampleFrames >= 0)
    {
		inleft  = (*in1++);
		inright = (*in2++);
		distorsio = iDistorsio;

		if (inleft > distorsio) {
			inleft=distorsio;
		}
		else if (inleft < -distorsio) {
			inleft = -distorsio;
		}

		if (inright > distorsio) {
			inright=distorsio;
		}
		else if (inright < -distorsio ) {
			inright = -distorsio;
		}


		*out1++ = inleft*fGain;
		*out2++=  inright*fGain;

    }
}


void ADistorsio::processDoubleReplacing (double** inputs, double** outputs, VstInt32 sampleFrames)
{
    double* in1  =  inputs[0];
    double* in2  =  inputs[1];
    double* out1 = outputs[0];
    double* out2 = outputs[1];

    while (--sampleFrames >= 0)
    {
		double inleft = (*in1++);
		double inright = (*in2++);

		double distorsio= double(iDistorsio);

		if (inleft > distorsio) {
			inleft=distorsio;
		}
		else if (inleft < -distorsio) {
			inleft = -distorsio;
		}

		if (inright > distorsio) {
			inright=distorsio;
		}
		else if (inright < -distorsio ) {
			inright = -distorsio;
		}


		*out1++ = inleft*fGain;
		*out2++=  inright*fGain;

    }
}