//--------------------------------------
// VST Plug-Ins SDK 
// ECO
//
// Francisco Javier Ripoll Esteve
// ripollete@hotmail.com
// 13/05/2008
//--------------------------------------

#include "eco.h"
#include "math.h"


AudioEffect* createEffectInstance (audioMasterCallback audioMaster) {
	return new AEco (audioMaster);
}


AEco::AEco (audioMasterCallback audioMaster) : AudioEffectX (audioMaster, 1, 3)	
{
	L = 44100*4;
	buffer1 = new float[L];
	buffer2 = new float[L];
	buffer1d = new double[L];
	buffer2d = new double[L];
		
	fDelay = 0.5;
	fDryWet = 0;
	fFeed = 0;

	posicion = 0;

	delay = int(L*fDelay);


	setNumInputs (2);	// stereo input
	setNumOutputs (2);	// stereo output

	setUniqueID ('ADly');

	resume ();		// flush buffer
}


AEco::~AEco () {
	if (buffer1)
		delete[] buffer1;

	if (buffer2)
		delete[] buffer2;
}


void AEco::setDelay (float value) {
	 fDelay = float(0.002*exp(value*log(4.0/0.002)));
	 delay= int(fDelay*L/4);
}


void AEco::resume () {
	memset (buffer1, 0, L * sizeof (float));
	memset (buffer2, 0, L * sizeof (float));
	AudioEffectX::resume ();
}


void AEco::setParameter (VstInt32 index, float value) {
	
	switch (index)
	{
		case kDelay  : setDelay(value);			break;				
		case kDryWet : fDryWet = (value*100);	break;
		case kFeed   : fFeed = (value*100);		break;
	}
}


float AEco::getParameter (VstInt32 index)
{
		float v = 0;

	switch (index) 	{
		case kDelay  : v = float(log(fDelay/0.002)/(log(4.0/0.002)));	break;
		case kDryWet : v = fDryWet/100;									break;
		case kFeed   : v = fFeed/100;									break;
	}
	return v;
}


void AEco::getParameterName (VstInt32 index, char* label) {
	switch (index) 	{
	case kDelay      : strcpy (label, "Delay");			break;
		case kDryWet : strcpy (label, "Dry -- Wet");	break;
		case kFeed   : strcpy (label, "FeedBack");		break;
	}
}


void AEco::getParameterDisplay (VstInt32 index, char* text) {
		switch (index) 	{
		case kDelay  : float2string (fDelay, text, kVstMaxParamStrLen);		break;
		case kDryWet : float2string (fDryWet, text, kVstMaxParamStrLen);	break;
		case kFeed   : float2string (fFeed, text, kVstMaxParamStrLen);		break;
	}
}


void AEco::getParameterLabel (VstInt32 index, char* label) {
	switch (index) 	{
		case kDelay  : strcpy (label, "segons");	break;
		case kDryWet : strcpy (label, "%");			break;
		case kFeed   : strcpy (label, "%");			break;
	}
}


bool AEco::getEffectName (char* name) {
	strcpy (name, "Eco");
	return true;
}


bool AEco::getProductString (char* text) {
	strcpy (text, "Eco");
	return true;
}


bool AEco::getVendorString (char* text) {
	strcpy (text, "Francisco Javier Ripoll Esteve");
	return true;
}


void AEco::processReplacing (float** inputs, float** outputs, VstInt32 sampleFrames) {
	float* in1 = inputs[0];
	float* in2 = inputs[1];
	float* out1 = outputs[0];
	float* out2 = outputs[1];

	while (--sampleFrames >= 0)	{

		//***** Guardem les entrades en variables*****//
		float inleft = (*in1++);
		float inright = (*in2++);


		//***** Calcul de la posici on es trobar el frame del retard *****//
		int posicionretardo = posicion-delay;

		if (posicionretardo<0) {
			posicionretardo= L+posicionretardo;
		}


		//**** Reproducci amb el in actual i el buffer(posici-retard) ****//

		float outleft = float(inleft*(1-(fDryWet/100))+buffer1[posicionretardo]*(fDryWet/100));
		float outright = float(inright*(1-(fDryWet/100))+buffer2[posicionretardo]*(fDryWet/100));
		*out1++ = outleft;
		*out2++=  outright;

	
		//***** Guardem el in actual en el buffer actual *****//
		buffer1[posicion]=(1-(fFeed/100))*inleft+(fFeed/100)*outleft;
		buffer2[posicion]=(1-(fFeed/100))*inright+(fFeed/100)*outright;


	
		posicion++;

		if (posicion==L) {
			posicion=0;
		}
	}
}


void AEco::processDoubleReplacing (double** inputs, double** outputs, VstInt32 sampleFrames) {
	double* in1 = inputs[0];
	double* in2 = inputs[1];
	double* out1 = outputs[0];
	double* out2 = outputs[1];

	while (--sampleFrames >= 0)	{

		
		//***** Guardem les entrades en variables*****//
		double inleft = (*in1++);
		double inright = (*in2++);


		//***** Calcul de la posici on es trobar el frame del retard *****//
		int posicionretardo = posicion-delay;

		if (posicionretardo<0) {
			posicionretardo= L+posicionretardo;
		}


		//**** Reproducci amb el in actual i el buffer(posici-retard) ****//

		double outleft = double(inleft*(1.0-(fDryWet/100.0))+buffer1d[posicionretardo]*(fDryWet/100.0));
		double outright = double(inright*(1.0-(fDryWet/100.0))+buffer2d[posicionretardo]*(fDryWet/100.0));
		*out1++ = outleft;
		*out2++=  outright;

	
		//***** Guardem el in actual en el buffer actual *****//
		
		buffer1d[posicion]=(1.0-(fFeed/100.0))*inleft+(fFeed/100.0)*outleft;
		buffer2d[posicion]=(1-(fFeed/100.0))*inright+(fFeed/100.0)*outright;


	
		posicion++;

		if (posicion==L) {
			posicion=0;
		}
	}
}