//--------------------------------------
// VST Plug-Ins SDK 
// GENERADOR
//
// Francisco Javier Ripoll Esteve
// ripollete@hotmail.com
// 13/05/2008
//--------------------------------------

#include "generador.h"
#include "math.h"


AudioEffect* createEffectInstance (audioMasterCallback audioMaster) {
	return new AGenerador (audioMaster);
}


AGenerador::AGenerador (audioMasterCallback audioMaster) : AudioEffectX (audioMaster, 1, 3) {
			
	f = AudioEffectX::updateSampleRate ();
	fAmplitud = 0.5;
	fFrequencia = 1000/f;
	fForma = FormaSinus;
	tipus = 0;
	i=0;
	
	pi = 3.141592653589793238462643383f;

	setNumOutputs (1);	// mono

	setUniqueID ('AGen');

	resume ();		// flush buffer
}


AGenerador::~AGenerador () {

}


void AGenerador::resume () {
	AudioEffectX::resume ();
}


void AGenerador::setParameter (VstInt32 index, float value) {	
	switch (index) 	{
		case kFrequencia : fFrequencia = FreqMin * exp(log(FreqMax/FreqMin)*value) / f;	break;				
		case kAmplitud   : fAmplitud = value;											break;
		case kForma      : fForma = int (float (NombreDeFormes - 1) * value );          break;
	}
}


float AGenerador::getParameter (VstInt32 index) {
	float v = 0;
	switch (index) 	{
		case kFrequencia : v = log(fFrequencia*f/FreqMin) / log (FreqMax/FreqMin);	break;
		case kAmplitud   : v = fAmplitud;	break;
		case kForma      : v = float (fForma) / float (NombreDeFormes - 1);
	}
	return v;
}


void AGenerador::getParameterName (VstInt32 index, char* label)
{
	switch (index) {
		case kFrequencia : strcpy (label, "Freqencia");		break;
		case kAmplitud   : strcpy (label, "Amplitud");			break;
		case kForma      : strcpy (label, "Forma de senyal");	break; 
	}
}


void AGenerador::getParameterDisplay (VstInt32 index, char* text)
{
		switch (index) {
			case kFrequencia : float2string (fFrequencia*f, text, kVstMaxParamStrLen);		break;
			case kAmplitud   : float2string (fAmplitud*100, text, kVstMaxParamStrLen);		break;
			case kForma      : switch (fForma) {
								case FormaSinus    : vst_strncpy (text, "Sinus", kVstMaxParamStrLen);    break;
								case FormaQuadra   : vst_strncpy (text, "Quadrat", kVstMaxParamStrLen);  break;
								case FormaTriangle : vst_strncpy (text, "Triangle", kVstMaxParamStrLen); break;
							   } break;
	}
}


void AGenerador::getParameterLabel (VstInt32 index, char* label) {
	switch (index) {
		case kFrequencia : strcpy (label, "Hz");	break;
		case kAmplitud   : strcpy (label, "%");		break;
		case kForma      : strcpy (label, "Tipus"); break;
	}
}


bool AGenerador::getEffectName (char* name) {
	strcpy (name, "Generador");
	return true;
}


bool AGenerador::getProductString (char* text) {
	strcpy (text, "Generador");
	return true;
}


bool AGenerador::getVendorString (char* text) {
	strcpy (text, "Francisco Javier Ripoll Esteve");
	return true;
}


void AGenerador::processReplacing (float** inputs, float** outputs, VstInt32 sampleFrames) {
	
	float* out = outputs[0];
	double fase;
	tipus =1;

	while (--sampleFrames >= 0)	{	
		fase = fmod(fFrequencia*i,1);
		i++;


		switch (fForma) {
		case FormaSinus :
			*out++ = fAmplitud*sin(2*pi*fase);
			break;
		case FormaQuadra :
			if(fase<0.5) {
				*out++ = fAmplitud;
			}
			else {
				*out++ = -fAmplitud;
			}
			break;
		case FormaTriangle :
			if(fase <= 0.5) {
				// y = m*x + b
				// m= DY / DX    DY = 2*Amplitud    DX = 1/2
				// x = fase
				// b = -Amplitud
				float m = 2 * fAmplitud / 0.5f;
				*out++ = m * fase - fAmplitud;
			}
			else {
				// y = m*x + b
				// m= DY / DX    DY = -2*Amplitud    DX = 1/2
				// x = (fase-0.5)   per a dur-ho a x=0
				// b = Amplitud

				float m = -2 * fAmplitud / 0.5f;
				*out++ = m * (fase - 0.5f) + fAmplitud;
				
			}

			break;

		}  //switch
	}
	
}
