//--------------------------------------
// VST Plug-Ins SDK 
// SINTETISADOR
//
// Francisco Javier Ripoll Esteve
// ripollete@hotmail.com
// 18/06/2008
//--------------------------------------

#include "sintetisador.h"
#include "math.h"
#include <stdio.h>


AudioEffect* createEffectInstance (audioMasterCallback audioMaster) {
	return new ASinteti (audioMaster);
}


void ASinteti::initProcess () {

	//Inicialitzem totes les posicions en valors inicials
	for(int pos=0; pos<85; pos++) {
		nota[pos].noteIsOn = false;
		nota[pos].estat = EOff;
		nota[pos].CAttack = 0;
		nota[pos].CDecay = 0;
		nota[pos].CRelease = 0;
		nota[pos].fPhase1 = 0;
		nota[pos].fPhase2 = 0;
		nota[pos].Delta = nota[pos].Velocity = 0;
		nota[pos].NoteFreq = conversorPosicionenFrecuencia(pos);
		nota[pos].salida = 0.0f;
	}

	f = AudioEffectX::updateSampleRate ();
	fForma = FormaSinus;
	i=0;
	
	TAttack = 22000.f;
	TDecay = 22000.f;
	TRelease = 22000.f;
	PSustain = 0.5f;
	fHeadRoom = 0.5f;
	volum = 1.0f;
	TeclesMaximes = 5;
	TTeclesMaximes = 0.5f;
	TeclesActives = 0;
	
	pi = 3.141592653589793238462643383f;

}


ASinteti::ASinteti (audioMasterCallback audioMaster) : AudioEffectX (audioMaster, 1, 6) {
	if (audioMaster) {		
		setNumInputs (0);
		setNumOutputs (1);
		canProcessReplacing ();
		isSynth ();
		setUniqueID ('Sint');
	}

	initProcess ();
	suspend ();
}


ASinteti::~ASinteti () {

}


void ASinteti::resume () {
	AudioEffectX::resume ();
}


void ASinteti::setParameter (VstInt32 index, float value) {	
	switch (index) 	{
		case kForma      : fForma = int (float (NombreDeFormes - 1) * value );	break;
		case kAttack	 : TAttack = value * f;									break;
		case kDecay		 : TDecay = value * f;									break;
		case kRelease	 : TRelease = value * f;								break;
		case kSustain    : PSustain = value;									break;
		case kTecles     : TTeclesMaximes = value;
							TeclesMaximes = int(TTeclesMaximes*10.0f);			break;
	}
}


float ASinteti::getParameter (VstInt32 index) {
	float v = 0;
	switch (index) 	{
		case kForma      : v = float (fForma) / float (NombreDeFormes - 1);		break;
		case kAttack	 : v = TAttack / f;										break;
		case kDecay		 : v = TDecay / f;										break;
		case kRelease	 : v = TRelease / f;									break;
		case kSustain    : v = PSustain;										break;
		case kTecles   : v = TTeclesMaximes;									break;
	}
	return v;
}


void ASinteti::getParameterName (VstInt32 index, char* label)
{
	switch (index) {
		case kForma      : strcpy (label, "Forma de senyal");	break; 
		case kAttack	 : strcpy (label, "Temps Attack");	    break;	
		case kDecay		 : strcpy (label, "Temps Decay");		break;
		case kRelease	 : strcpy (label, "Temps Release");		break;
		case kSustain    : strcpy (label, "Sustain");			break;
		case kTecles   : strcpy (label, "TeclesMaximes");			break;
	}
}


void ASinteti::getParameterDisplay (VstInt32 index, char* text)
{
		switch (index) {
			case kForma      : switch (fForma) {
								case FormaSinus    : vst_strncpy (text, "Sinus", kVstMaxParamStrLen);    break;
								case FormaQuadra   : vst_strncpy (text, "Quadrat", kVstMaxParamStrLen);  break;
								case FormaTriangle : vst_strncpy (text, "Triangle", kVstMaxParamStrLen); break;
								//case FormaSinus : vst_strncpy (text, "Sinus", kVstMaxParamStrLen); break;
							   } break;
			case kAttack	 : float2string (TAttack/f, text, kVstMaxParamStrLen);    break;	
			case kDecay		 : float2string (TDecay/f, text, kVstMaxParamStrLen);     break;
			case kRelease	 : float2string (TRelease/f, text, kVstMaxParamStrLen);   break;
			case kSustain    : float2string (PSustain, text, kVstMaxParamStrLen);     break;
			case kTecles   : int2string    (TeclesMaximes, text, kVstMaxParamStrLen);	  break;
	}
}


void ASinteti::getParameterLabel (VstInt32 index, char* label) {
	switch (index) {
		case kForma      : strcpy (label, "Tipus");		break;
		case kAttack	 : strcpy (label, "ms");		break;	
		case kDecay		 : strcpy (label, "ms");		break;
		case kRelease	 : strcpy (label, "ms");		break;
		case kSustain    : strcpy (label, "%");			break;
		case kTecles   : strcpy (label, "Tecles");		break;
	}
}


bool ASinteti::getEffectName (char* name) {
	strcpy (name, "Generador");
	return true;
}


bool ASinteti::getProductString (char* text) {
	strcpy (text, "Generador");
	return true;
}


bool ASinteti::getVendorString (char* text) {
	strcpy (text, "Francisco Javier Ripoll Esteve");
	return true;
}


float ASinteti::conversorPosicionenFrecuencia(int pos) {
	if (pos == 0) return 32.71f; // C octava1
	else if (pos == 1) return 34.65f;
	else if (pos == 2) return 36.71f;
	else if (pos == 3) return 38.89f;
	else if (pos == 4) return 41.20f;
	else if (pos == 5) return 43.65f;
	else if (pos == 6) return 46.25f;
	else if (pos == 7) return 49.0f;
	else if (pos == 8) return 51.91f;
	else if (pos == 9) return 55.0f;
	else if (pos == 10) return 58.27f;
	else if (pos == 11) return 61.74f;
	else if (pos == 12) return 65.41f; // C octava2
	else if (pos == 13) return 69.30f;
	else if (pos == 14) return 73.42f;
	else if (pos == 15) return 77.78f;
	else if (pos == 16) return 82.41f;
	else if (pos == 17) return 87.31f;
	else if (pos == 18) return 92.50f;
	else if (pos == 19) return 98.0f;
	else if (pos == 20) return 103.83f;
	else if (pos == 21) return 110.0f;
	else if (pos == 22) return 116.54f;
	else if (pos == 23) return 123.47f;
	else if (pos == 24) return 130.81f; // C octava 3
	else if (pos == 25) return 138.59f;
	else if (pos == 26) return 146.83f;
	else if (pos == 27) return 155.56f;
	else if (pos == 28) return 164.81f;
	else if (pos == 29) return 174.61f;
	else if (pos == 30) return 185.0f;
	else if (pos == 31) return 196.0f;
	else if (pos == 32) return 207.65f;
	else if (pos == 33) return 220.0f;
	else if (pos == 34) return 233.08f;
	else if (pos == 35) return 246.94f;
	else if (pos == 36) return 261.63f;  // C octava 4
	else if (pos == 37) return 277.18f;
	else if (pos == 38) return 293.66f;
	else if (pos == 39) return 311.13f;
	else if (pos == 40) return 329.63f;
	else if (pos == 41) return 349.23f;
	else if (pos == 42) return 369.99f;
	else if (pos == 43) return 392.0f;
	else if (pos == 44) return 415.30f;
	else if (pos == 45) return 444.0f;
	else if (pos == 46) return 466.16f;
	else if (pos == 47) return 493.88f;
	else if (pos == 48) return 523.25f;  // C octava 5
	else if (pos == 49) return 554.37f;
	else if (pos == 50) return 587.33f;
	else if (pos == 51) return 622.25f;
	else if (pos == 52) return 659.26f;
	else if (pos == 53) return 698.46f;
	else if (pos == 54) return 739.99f;
	else if (pos == 55) return 783.99f;
	else if (pos == 56) return 830.61f;
	else if (pos == 57) return 880.0f;
	else if (pos == 58) return 932.33f;
	else if (pos == 59) return 987.77f;
	else if (pos == 60) return 1046.50f; // C octava 6
	else if (pos == 61) return 1108.73f;
	else if (pos == 62) return 1174.66f;
	else if (pos == 63) return 1244.51f;
	else if (pos == 64) return 1318.51f;
	else if (pos == 65) return 1396.91f;
	else if (pos == 66) return 1479.98f;
	else if (pos == 67) return 1567.98f;
	else if (pos == 68) return 1661.22f;
	else if (pos == 69) return 1760.0f;
	else if (pos == 70) return 1864.66f;
	else if (pos == 71) return 1975.53f;
	else if (pos == 72) return 2093.0f;
	else if (pos == 73) return 2217.46f;
	else if (pos == 74) return 2349.32f;
	else if (pos == 75) return 2489.02f;
	else if (pos == 76) return 2637.02f;
	else if (pos == 77) return 2793.83f;
	else if (pos == 78) return 2959.96f;
	else if (pos == 79) return 3135.96f;
	else if (pos == 80) return 3322.44f;
	else if (pos == 81) return 3520.0f;
	else if (pos == 82) return 3729.31f;
	else if (pos == 83) return 3951.07f;
	else if (pos == 84) return 4186.01f; // C octava 7


	
}


int ASinteti::conversorNotaenPosicion(VstInt32 Nota) {
	if (Nota == 0x18) return 0; // C octava1
	else if (Nota == 0x19) return 1;
	else if (Nota == 0x1A) return 2;
	else if (Nota == 0x1B) return 3;
	else if (Nota == 0x1C) return 4;
	else if (Nota == 0x1D) return 5;
	else if (Nota == 0x1E) return 6;
	else if (Nota == 0x1F) return 7;
	else if (Nota == 0x20) return 8;
	else if (Nota == 0x21) return 9;
	else if (Nota == 0x22) return 10;
	else if (Nota == 0x23) return 11;
	else if (Nota == 0x24) return 12; // C octava2
	else if (Nota == 0x25) return 13;
	else if (Nota == 0x26) return 14;
	else if (Nota == 0x27) return 15;
	else if (Nota == 0x28) return 16;
	else if (Nota == 0x29) return 17;
	else if (Nota == 0x2A) return 18;
	else if (Nota == 0x2B) return 19;
	else if (Nota == 0x2C) return 20;
	else if (Nota == 0x2D) return 21;
	else if (Nota == 0x2E) return 22;
	else if (Nota == 0x2F) return 23;
	else if (Nota == 0x30) return 24; // C octava 3
	else if (Nota == 0x31) return 25;
	else if (Nota == 0x32) return 26;
	else if (Nota == 0x33) return 27;
	else if (Nota == 0x34) return 28;
	else if (Nota == 0x35) return 29;
	else if (Nota == 0x36) return 30;
	else if (Nota == 0x37) return 31;
	else if (Nota == 0x38) return 32;
	else if (Nota == 0x39) return 33;
	else if (Nota == 0x3A) return 34;
	else if (Nota == 0x3B) return 35;
	else if (Nota == 0x3C) return 36;  // C octava 4
	else if (Nota == 0x3D) return 37;
	else if (Nota == 0x3E) return 38;
	else if (Nota == 0x3F) return 39;
	else if (Nota == 0x40) return 40;
	else if (Nota == 0x41) return 41;
	else if (Nota == 0x42) return 42;
	else if (Nota == 0x43) return 43;
	else if (Nota == 0x44) return 44;
	else if (Nota == 0x45) return 45;
	else if (Nota == 0x46) return 46;
	else if (Nota == 0x47) return 47;
	else if (Nota == 0x48) return 48;  // C octava 5
	else if (Nota == 0x49) return 49;
	else if (Nota == 0x4A) return 50;
	else if (Nota == 0x4B) return 51;
	else if (Nota == 0x4C) return 52;
	else if (Nota == 0x4D) return 53;
	else if (Nota == 0x4E) return 54;
	else if (Nota == 0x4F) return 55;
	else if (Nota == 0x50) return 56;
	else if (Nota == 0x51) return 57;
	else if (Nota == 0x52) return 58;
	else if (Nota == 0x53) return 59;
	else if (Nota == 0x54) return 60; // C octava 6
	else if (Nota == 0x55) return 61;
	else if (Nota == 0x56) return 62;
	else if (Nota == 0x57) return 63;
	else if (Nota == 0x58) return 64;
	else if (Nota == 0x59) return 65;
	else if (Nota == 0x5A) return 66;
	else if (Nota == 0x5B) return 67;
	else if (Nota == 0x5C) return 68;
	else if (Nota == 0x5D) return 69;
	else if (Nota == 0x5E) return 70;
	else if (Nota == 0x5F) return 71;
	else if (Nota == 0x60) return 72;
	else if (Nota == 0x61) return 73;
	else if (Nota == 0x62) return 74;
	else if (Nota == 0x63) return 75;
	else if (Nota == 0x64) return 76;
	else if (Nota == 0x65) return 77;
	else if (Nota == 0x66) return 78;
	else if (Nota == 0x67) return 79;
	else if (Nota == 0x68) return 80;
	else if (Nota == 0x69) return 81;
	else if (Nota == 0x6A) return 82;
	else if (Nota == 0x6B) return 83;
	else if (Nota == 0x6C) return 84; // C octava 7
}
	

void ASinteti::processReplacing (float** inputs, float** outputs, VstInt32 sampleFrames) {
	
	float* out = outputs[0];
	double fase;
	float eixida;
	float PAttack; //Percentatge Attack
	float PDecay;
	float PRelease;
	float fFrequencia;
	float fAmplitud;
	
	while (--sampleFrames >= 0)	{
		for(int pos=0;pos<85;pos++){
			if(nota[pos].noteIsOn) {
				
				fFrequencia = nota[pos].NoteFreq / f;
				fase = fmod(fFrequencia*i,1);
				fAmplitud = double(nota[pos].Velocity)/127.0f;
				
				switch (fForma) {
					case FormaSinus :
						nota[pos].salida = fAmplitud*sin(2*pi*fase);
						break;
					case FormaQuadra :
						if(fase<0.5) {
							nota[pos].salida = fAmplitud;
						}
						else {
							nota[pos].salida = -fAmplitud;
						}
						break;
					case FormaTriangle :
						if(fase <= 0.5) {
							// y = m*x + b
							// m= DY / DX    DY = 2*Amplitud    DX = 1/2
							// x = fase
							// b = -Amplitud
							float m = 2 * fAmplitud / 0.5f;
							nota[pos].salida = m * fase - fAmplitud;
						}
						else {
							// y = m*x + b
							// m= DY / DX    DY = -2*Amplitud    DX = 1/2
							// x = (fase-0.5)   per a dur-ho a x=0
							// b = Amplitud

							float m = -2 * fAmplitud / 0.5f;
							nota[pos].salida = m * (fase - 0.5f) + fAmplitud;
							
						} 
						break;
				}  //fi switch (fForma)

				switch (nota[pos].estat) {
					case EAttack:
						PAttack = float(nota[pos].CAttack) / TAttack;
						nota[pos].salida = nota[pos].salida * PAttack;
						nota[pos].CAttack++;
						if (nota[pos].CAttack >= TAttack) {
							nota[pos].estat = EDecay;
						}
						break;
					case EDecay:
						//                 DY         DX        x                        b
						PDecay = ((PSustain-1.0f) / TDecay) * float(nota[pos].CDecay) + 1.0f;
						nota[pos].salida = nota[pos].salida * PDecay;
						nota[pos].CDecay++;
						if (nota[pos].CDecay >= TDecay) {
							nota[pos].estat = ESustain;
						}
						break;
					case ESustain:
						nota[pos].salida = nota[pos].salida*PSustain;
						break;
					case ERelease:
						PRelease = (-PSustain / TRelease) * float(nota[pos].CRelease) + PSustain;
						nota[pos].salida = nota[pos].salida*PRelease;
						nota[pos].CRelease++;
						if (nota[pos].CRelease >= TRelease) {
							nota[pos].estat= EOff;
							nota[pos].noteIsOn = false;
						}
						break;
				}
			} // fi if(nota[pos].noteIsOn)
			else nota[pos].salida = 0.0f;
		} // fi for(int pos=0;pos<85;pos++)
		
		float exit = 0.0f;

		// Suma de totes les notes que estan actives
		for(pos=0;pos<85;pos++) {	
			if (nota[pos].noteIsOn) {
				exit = nota[pos].salida*1/TeclesMaximes + exit;

			}
		}
		*out++ = exit * (volum*2);
		i++;


	} // fi while
	
}




VstInt32 ASinteti::processEvents (VstEvents* ev) {
	for (VstInt32 i = 0; i < ev->numEvents; i++) {
		if ((ev->events[i])->type != kVstMidiType)
			continue;

		VstMidiEvent* event = (VstMidiEvent*)ev->events[i];
		char* midiData = event->midiData;
		VstInt32 status = midiData[0] & 0xf0;	// Seleccionem els 4 primers bits
		if (status == 0x90 || status == 0x80) {	// Comparem que els 4 primers bits siguen activar o desactivar nota
			VstInt32 note = midiData[1] & 0x7f;
			VstInt32 velocity = midiData[2] & 0x7f;
			if (status == 0x80) 
				noteOff (note);
			else {
				if (TeclesActives < TeclesMaximes)
					noteOn (note, velocity, event->deltaFrames);
			}
		}
		else if (status == 0xb0) {
			if (midiData[1] == 0x7e || midiData[1] == 0x7b)	// totes les notes off
				noteOffAll();
			if (midiData[1] == 0x48) {
				int midiData2 = midiData[2];
				setParameter(kRelease,midiData2/127.0f);
				getParameter(kRelease);
			}
			if (midiData[1] == 0x49) {
				int midiData2 = midiData[2];
				setParameter(kAttack,midiData2/127.0f);
				getParameter(kAttack);
			}
			
			if (midiData[1] == 0x4B) {
				int midiData2 = midiData[2];
				setParameter(kDecay,midiData2/127.0f);
				getParameter(kDecay);
			}
			if (midiData[1] == 0x4F) {
				int midiData2 = midiData[2];
				setParameter(kSustain,midiData2/127.0f);
				getParameter(kSustain);
			}
			if (midiData[1] == 0x07) {
				int midiData2 = midiData[2];
				volum = midiData2/127.0f;

			}
		}
		event++;
	}
	return 1;
}


void ASinteti::noteOn (VstInt32 note, VstInt32 velocity, VstInt32 delta) {
	int pos;
	pos = conversorNotaenPosicion(note);
	nota[pos].Velocity = velocity;
	nota[pos].Delta = delta;
	nota[pos].noteIsOn = true;
	nota[pos].fPhase1 = nota[pos].fPhase2 = 0;
	nota[pos].CAttack = nota[pos].CDecay = nota[pos].CRelease = 0;
	nota[pos].salida = 0.0f;
	nota[pos].estat = EAttack;
	TeclesActives++;
}


void ASinteti::noteOff (VstInt32 note) {
	int pos;
	pos = pos = conversorNotaenPosicion(note);
	nota[pos].estat = ERelease;
	if(nota[pos].noteIsOn == true)
		TeclesActives--;
}

void ASinteti::noteOffAll() {
	for(int pos=0; pos<85; pos++) {
		nota[pos].noteIsOn = false;
		nota[pos].estat = EOff;
		nota[pos].CAttack = 0;
		nota[pos].CDecay = 0;
		nota[pos].CRelease = 0;
		nota[pos].fPhase1 = 0;
		nota[pos].fPhase2 = 0;
		nota[pos].Delta = nota[pos].Velocity = 0;
		nota[pos].NoteFreq = conversorPosicionenFrecuencia(pos);
		nota[pos].salida = 0.0f;
	}
}
